"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildResult = buildResult;
exports.default = configure;
exports.DEFAULTS = void 0;

var _viswizSdk = _interopRequireDefault(require("viswiz-sdk"));

var _utils = require("../utils");

var validators = _interopRequireWildcard(require("../validators"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint no-console: 0 */
const DEFAULTS = {
  POLL_SECONDS: 5,
  WAIT_SECONDS: 600
};
exports.DEFAULTS = DEFAULTS;

async function waitForResult(client, projectID, buildID, maxTime) {
  let timeoutTimer;
  return Promise.race([new Promise((resolve, reject) => {
    timeoutTimer = setTimeout(() => reject(new Error('Waiting for results timed out!')), maxTime * 1000);
  }), new Promise(resolve => {
    let time = 0;

    function poll() {
      (0, _utils.log)(`Waiting for results to be ready... (${(0, _utils.formatTime)(time)}/${(0, _utils.formatTime)(maxTime)})`);
      client.getBuilds(projectID).then(builds => {
        const build = builds.find(item => item.id === buildID);

        if (!build || !build.diffedAt) {
          return schedule();
        }

        clearTimeout(timeoutTimer);
        resolve(build.diffPercentage);
      }).catch(schedule);
    }

    function schedule() {
      time += DEFAULTS.POLL_SECONDS;
      setTimeout(poll, DEFAULTS.POLL_SECONDS * 1000);
    }

    poll();
  })]);
}

function finalize(buildDiffPercentage, project) {
  const passed = buildDiffPercentage <= project.diffThreshold;
  (0, _utils.log)(`Build difference: ${buildDiffPercentage}%`);
  (0, _utils.log)(`Project difference threshold: ${project.diffThreshold}%`);

  if (!passed) {
    (0, _utils.error)('Build failed!');
  } else {
    (0, _utils.log)('Build passed!');
  }
}

async function buildResult(program, options) {
  if (validators.apiKey(program) || validators.projectID(program)) {
    return;
  }

  const {
    apiKey,
    project: projectID
  } = program;
  let {
    build: buildID
  } = options;
  const client = new _viswizSdk.default(apiKey);
  const [project, builds] = await Promise.all([client.getProject(projectID), client.getBuilds(projectID)]);

  if (!builds.length) {
    return (0, _utils.error)('Error: The project does not have any builds!', program);
  }

  if (builds.length === 1) {
    (0, _utils.log)('This is the first build for this project. No comparison available.');
    return 'OK';
  }

  let build;

  if (buildID) {
    build = builds.find(item => item.id === buildID);
  } else {
    build = builds[0];
    buildID = build.id;
  }

  if (options.waitForResult) {
    const buildDiffPercentage = await waitForResult(client, projectID, buildID, typeof options.waitForResult === 'string' ? parseInt(options.waitForResult, 10) : DEFAULTS.WAIT_SECONDS);
    (0, _utils.log)('');
    finalize(buildDiffPercentage, project);
  } else {
    finalize(build.diffPercentage, project);
  }

  return 'OK';
}

function configure(program) {
  program.command('build-result').description('Gets or waits for a build result').option('-b, --build [buildID]', 'The build ID to get results for. If not sent, then the most recent build for the project is used.').option('-w, --wait-for-result [timeout]', 'Whether to wait for the result of the build comparison (disabled by default). Waits for a maximum number of seconds (defaults to 600).').action((cmd, options) => buildResult(program, options || cmd).catch(err => {
    (0, _utils.error)(`Error: ${err.message}`);
  }));
}